unit ShadeHeightImgForm;

{
Revision history:

1.0.0  2000 Apr 13  First version
1.1.0  2000 Apr 26  Move to mapping folder
                    Rename to ShadeHeightImg
1.2.0  2000 May 22  Add menu view of cell statistics
                    Suggest JPEG image is saved with a compression of 30
1.2.2  2000 May 27  Add check for PNG file
                    Ask user through File Open dialog if not found
1.2.4  2000 May 28  Add Roger Hedin's sea colour choice
                    Correct failure to write bottom and right pixel rows
1.3.0  2001 Sep 21  Version for Delphi 5
                    Alter height interpretation to H = 3 * sqrt (metres)
                    Tweak sea colour to 90% of previous values
                    Use PNG_IO unit
1.3.2  2002 Jan 24  Remove dependancy on TExtBitmap in BMPExtImage unit
                    Add visual feedback of progress

Copyright  David J Taylor, Edinburgh
Web site:  www.satsignal.net
E-mail:    davidtaylor@writeme.com
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Menus, PNG_IO, ComCtrls, ElevationColours, StdCtrls;

const
  sea = 231;
  grey = 128;
  _clSea1: TRGBTriple = (rgbtBlue: sea;  rgbtGreen: sea;  rgbtRed: sea);
  _clSea0: TRGBTriple = (rgbtBlue: 220;  rgbtGreen: 200;  rgbtRed: 150);
  clSea: TRGBTriple = (rgbtBlue: 198;  rgbtGreen: 180;  rgbtRed: 135);

type
  TFormMain = class(TForm)
    MainMenu1: TMainMenu;
    MenuFile: TMenuItem;
    FileExit: TMenuItem;
    MenuHelp: TMenuItem;
    HelpAbout: TMenuItem;
    TimerOnShow: TTimer;
    TabControlImage: TTabControl;
    ImageDisplay: TImage;
    MenuView: TMenuItem;
    ViewCellStatistics: TMenuItem;
    OpenDialog: TOpenDialog;
    ProgressBarStep: TProgressBar;
    ProgressBarPercent: TProgressBar;
    LabelStep: TLabel;
    LabelPercent: TLabel;
    procedure FileExitClick (Sender: TObject);
    procedure FormCreate (Sender: TObject);
    procedure FormDestroy (Sender: TObject);
    procedure FormShow (Sender: TObject);
    procedure HelpAboutClick (Sender: TObject);
    procedure TabControlImageChange (Sender: TObject);
    procedure TimerOnShowTimer (Sender: TObject);
    procedure ViewCellStatisticsClick (Sender: TObject);
  private
    { Private declarations }
    height_bmp: TBitmap;
    colour_bmp: TBitmap;
    relief_bmp: TBitmap;
    combined_bmp: TBitmap;
    cells: array [1..5] of integer;
    t0: DWORD;
    procedure make_colour;
    procedure make_greyscale (bmp: TBitmap);
    procedure make_relief;
    procedure make_combined;
    procedure update_progress (const percent: integer);
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;


implementation

uses
  AboutDisplay;

{$R *.DFM}

procedure TFormMain.FormCreate (Sender: TObject);
begin
  height_bmp := TBitmap.Create;
  height_bmp.PixelFormat := pf8bit;
  make_greyscale (height_bmp);
  colour_bmp := TBitmap.Create;
  colour_bmp.PixelFormat := pf24bit;
  relief_bmp := TBitmap.Create;
  relief_bmp.PixelFormat := pf8bit;
  make_greyscale (relief_bmp);
  combined_bmp := TBitmap.Create;
  combined_bmp.PixelFormat := pf24bit;
end;


procedure TFormMain.FormDestroy (Sender: TObject);
begin
  height_bmp.Free;
  colour_bmp.Free;
  relief_bmp.Free;
  combined_bmp.Free;
end;


procedure TFormMain.HelpAboutClick (Sender: TObject);
begin
  AboutBoxDJT1.ShowModal;
end;


procedure TFormMain.FileExitClick (Sender: TObject);
begin
  Close;
end;


procedure TFormMain.TimerOnShowTimer (Sender: TObject);
var
  png_filename: string;
  saved_cursor: TCursor;
begin
  TimerOnShow.Enabled := False;
  png_filename := 'topo.png';
  if not FileExists (png_filename) then
    if OpenDialog.Execute then png_filename := OpenDialog.Filename;
  if not FileExists (png_filename) then
    begin
    ShowMessage ('Cannot find height PNG file ' +
                 png_filename + ', so closing down.');
    Close;
    Exit;
    end;
  saved_cursor := Screen.Cursor;
  Screen.Cursor := crHourGlass;

  ProgressBarStep.Position := 0;
  LoadBmpFromPngFile (height_bmp, png_filename, False);
  ImageDisplay.Picture.Bitmap := height_bmp;
  TabControlImage.TabIndex := 0;
  TabControlImageChange (Self);

  ProgressBarStep.Position := 1;
  make_colour;
  TabControlImage.TabIndex := 1;
  TabControlImageChange (Self);

  ProgressBarStep.Position := 2;
  make_relief;
  TabControlImage.TabIndex := 2;
  TabControlImageChange (Self);

  ProgressBarStep.Position := 3;
  make_combined;
  ProgressBarPercent.Position := 0;
  TabControlImage.TabIndex := 3;
  TabControlImageChange (Self);

  ProgressBarStep.Position := 4;
  combined_bmp.SaveToFile ('combined.bmp');
  ProgressBarStep.Position := 0;
  ShowMessage ('Coloured, shaded image written to COMBINED.BMP');

  Screen.Cursor := saved_cursor;
end;


procedure TFormMain.FormShow (Sender: TObject);
begin
  TimerOnShow.Enabled := True;
end;


procedure TFormMain.TabControlImageChange (Sender: TObject);
begin
  case TabControlImage.TabIndex of
    0: ImageDisplay.Picture.Bitmap := height_bmp;
    1: ImageDisplay.Picture.Bitmap := colour_bmp;
    2: ImageDisplay.Picture.Bitmap := relief_bmp;
    3: ImageDisplay.Picture.Bitmap := combined_bmp;
  end;
  ImageDisplay.Update;
end;


procedure TFormMain.make_colour;
var
  h: integer;
  w: integer;
  row: integer;
  col: integer;
  pb: ^Byte;
  pc: ^RGBTriple;
  height: integer;
begin
  h := height_bmp.Height;
  w := height_bmp.Width;
  colour_bmp.Width := w;
  colour_bmp.Height := h;
  for row := 0 to h - 1 do
    begin
    pb := height_bmp.ScanLine [row];
    pc := colour_bmp.ScanLine [row];
    for col := 0 to w - 1 do
      begin
      height := pb^;
      if height = 0
      then
        pc^ := clSea
      else
        begin
//        height := (height - 1) * 25;
        height := Round (height * height / 9);
        pc^.rgbtBlue := colours [height] shr 16;
        pc^.rgbtGreen := (colours [height] and $FF00) shr 8;
        pc^.rgbtRed := colours [height] and $FF;
        end;
      Inc (pb);
      Inc (pc);
      end;
    update_progress (100 * row div h);
    end;
end;


procedure TFormMain.make_relief;
const
  adjusts: array [1..5] of integer = (5, 7, 10, 15, 20);
//  adjusts: array [1..5] of integer = (24, 32, 38, 42, 45);
var
  h: integer;
  w: integer;
  row: integer;
  col: integer;
  pb: ^Byte;
  pbb: ^Byte;
  pr: ^Byte;
  h1: integer;
  h2: integer;
  dh: integer;
  sea: byte;
  adjust: integer;
  cell: integer;
begin
  h := height_bmp.Height;
  w := height_bmp.Width;
  relief_bmp.Canvas.Brush.Color := RGB (grey, grey, grey);
  relief_bmp.Width := w;
  relief_bmp.Height := h;
  sea := height_bmp.Canvas.Pixels [0, 0];
  for cell := 1 to 5 do cells [cell] := 0;
  for row := 0 to h - 2 do
    begin
    pb := height_bmp.ScanLine [row];
    pbb := height_bmp.ScanLine [row + 1];
    Inc (pbb);
    pr := relief_bmp.ScanLine [row];
    for col := 0 to w - 2 do
      begin
      h1 := pb^;
      h2 := pbb^;
      adjust := 0;
      if (h1 <> sea) and (h2 <> sea) then
        begin
        dh := abs (h2 - h1);
        case dh of
          1..5: begin
                adjust := adjusts [dh];
                Inc (cells [dh]);
                end;
          6..32767: adjust := adjusts [5];
        else
          adjust := 0;
        end;
        if h1 > h2 then adjust := -adjust;
        end;
      pr^ := grey + adjust;
      Inc (pb);
      Inc (pbb);
      Inc (pr);
      end;
    update_progress (100 * row div h);
    end;
end;


procedure TFormMain.make_combined;
var
  h: integer;
  w: integer;
  row: integer;
  col: integer;
  pc: ^RGBTriple;
  pr: ^Byte;
  pcc: ^RGBTriple;
  r, g, b: integer;
  adjust: integer;
begin
  h := colour_bmp.Height;
  w := colour_bmp.Width;
  combined_bmp.Width := w;
  combined_bmp.Height := h;
  for row := 0 to h - 1 do
    begin
    pc := colour_bmp.ScanLine [row];
    pr := relief_bmp.ScanLine [row];
    pcc := combined_bmp.ScanLine [row];
    for col := 0 to w - 1 do
      begin
      b := pc^.rgbtBlue;
      g := pc^.rgbtGreen;
      r := pc^.rgbtRed;
      adjust := pr^ - grey;
      b := b + adjust;
      if b > 255 then b := 255;
      if b < 0 then b := 0;
      g := g + adjust;
      if g > 255 then g := 255;
      if g < 0 then g := 0;
      r := r + adjust;
      if r > 255 then r := 255;
      if r < 0 then r := 0;
      pcc^.rgbtBlue := b;
      pcc^.rgbtGreen := g;
      pcc^.rgbtRed := r;
      Inc (pc);
      Inc (pr);
      Inc (pcc);
      end;
    update_progress (100 * row div h);
    end;
end;


procedure TFormMain.make_greyscale (bmp: TBitmap);
var
  b: integer;
  log_pal: TMaxLogPalette;
  pal: HPalette;
begin
  with bmp do
    begin
    log_pal.palVersion := $0300;
    log_pal.palNumEntries := 256;
    for b := 0 to 255 do
      with log_pal.palPalEntry [b] do
        begin
        peBlue := b;
        peGreen := b;
        peRed := b;
        peFlags := 0;
        end;
    pal := CreatePalette (PLogPalette(@log_pal)^);
    Palette := pal;
    Modified := True;
    end;
end;


procedure TFormMain.ViewCellStatisticsClick (Sender: TObject);
var
  st: string;
  cell: integer;
begin
  st := 'Number of cells at each height difference' + #13;
  for cell := 1 to 5 do st := st + IntToStr (cells [cell]) + '  ';
  ShowMessage (st);
end;


procedure TFormMain.update_progress (const percent: integer);
begin
  if (GetTickCount - t0) > 100 then
    begin
    ProgressBarPercent.Position := percent;
    t0 := GetTickCount;
    end;
end;

end.

